@@ -10,13 +10,14 @@
 module Backend (
 	builtinList,
 	defaultBackend,
-	defaultHashBackend,
+	hashBackend,
 	genKey,
 	getBackend,
 	chooseBackend,
 	lookupBackendVariety,
 	lookupBuiltinBackendVariety,
 	maybeLookupBackendVariety,
+	unknownBackendVarietyMessage,
 	isStableKey,
 	isCryptographicallySecureKey,
 	isCryptographicallySecure,
@@ -54,6 +55,15 @@ defaultBackend = maybe cache return =<< Annex.getState Annex.backend
 	valid name = not (null name)
 	lookupname = lookupBackendVariety . parseKeyVariety . encodeBS
 
+{- A hashing backend. Takes git config into account, but
+ - guarantees the backend is cryptographically secure. -}
+hashBackend :: Annex Backend
+hashBackend = do
+	db <- defaultBackend
+	return $ if isCryptographicallySecure db
+		then db
+		else defaultHashBackend
+
 {- Generates a key for a file. -}
 genKey :: KeySource -> MeterUpdate -> Backend -> Annex (Key, Backend)
 genKey source meterupdate b = case B.genKey b of
@@ -63,11 +73,11 @@ genKey source meterupdate b = case B.genKey b of
 	Nothing -> giveup $ "Cannot generate a key for backend " ++
 		decodeBS (formatKeyVariety (B.backendVariety b))
 
-getBackend :: FilePath -> Key -> Annex (Maybe Backend)
+getBackend :: OsPath -> Key -> Annex (Maybe Backend)
 getBackend file k = maybeLookupBackendVariety (fromKey keyVariety k) >>= \case
 	Just backend -> return $ Just backend
 	Nothing -> do
-		warning $ "skipping " <> QuotedPath (toRawFilePath file) <> " (" <>
+		warning $ "skipping " <> QuotedPath file <> " (" <>
 			UnquotedString (unknownBackendVarietyMessage (fromKey keyVariety k)) <> ")"
 		return Nothing
 
@@ -78,7 +88,7 @@ unknownBackendVarietyMessage v =
 {- Looks up the backend that should be used for a file.
  - That can be configured on a per-file basis in the gitattributes file,
  - or forced with --backend. -}
-chooseBackend :: RawFilePath -> Annex Backend
+chooseBackend :: OsPath -> Annex Backend
 chooseBackend f = Annex.getRead Annex.forcebackend >>= go
   where
 	go Nothing = do
