@@ -6,17 +6,14 @@
  - Licensed under the GNU AGPL version 3 or higher.
  -}
 
+{-# LANGUAGE OverloadedStrings #-}
 {-# LANGUAGE CPP #-}
 {-# OPTIONS_GHC -fno-warn-tabs #-}
 
 module Build.DesktopFile where
 
-import Utility.Exception
+import Common
 import Utility.FreeDesktop
-import Utility.Path
-import Utility.Monad
-import Utility.SystemDirectory
-import Utility.FileSystemEncoding
 import Config.Files
 import Utility.OSX
 import Assistant.Install.AutoStart
@@ -25,8 +22,6 @@ import Assistant.Install.Menu
 import System.Environment
 #ifndef mingw32_HOST_OS 
 import System.Posix.User
-import Data.Maybe
-import Control.Applicative
 import Prelude
 #endif
 
@@ -42,10 +37,10 @@ systemwideInstall = isroot <||> (not <$> userdirset)
 systemwideInstall = return False
 #endif
 
-inDestDir :: FilePath -> IO FilePath
+inDestDir :: OsPath -> IO OsPath
 inDestDir f = do
 	destdir <- catchDefaultIO "" (getEnv "DESTDIR")
-	return $ destdir ++ "/" ++ f
+	return $ toOsPath destdir <> literalOsPath "/" <> f
 
 writeFDODesktop :: FilePath -> IO ()
 writeFDODesktop command = do
@@ -54,7 +49,7 @@ writeFDODesktop command = do
 	datadir <- if systemwide then return systemDataDir else userDataDir
 	menufile <- inDestDir (desktopMenuFilePath "git-annex" datadir)
 	icondir <- inDestDir (iconDir datadir)
-	installMenu command menufile "doc" icondir
+	installMenu command menufile (literalOsPath "doc") icondir
 
 	configdir <- if systemwide then return systemConfigDir else userConfigDir
 	installAutoStart command 
@@ -78,8 +73,8 @@ install command = do
 		( return ()
 		, do
 			programfile <- inDestDir =<< programFile
-			createDirectoryIfMissing True (fromRawFilePath (parentDir (toRawFilePath programfile)))
-			writeFile programfile command
+			createDirectoryIfMissing True (parentDir programfile)
+			writeFile (fromOsPath programfile) command
 		)
 
 installUser :: FilePath -> IO ()
