@@ -25,7 +25,7 @@ import Utility.Tmp.Dir
 import Utility.FileMode
 import Utility.ThreadScheduler
 import Utility.SafeOutput
-import qualified Utility.RawFilePath as R
+import qualified Utility.FileIO as F
 import qualified Utility.MagicWormhole as Wormhole
 
 import Control.Concurrent.Async
@@ -193,12 +193,11 @@ serializePairData :: PairData -> String
 serializePairData (PairData (HalfAuthToken ha) addrs) = unlines $
 	T.unpack ha : map formatP2PAddress addrs
 
-deserializePairData :: String -> Maybe PairData
-deserializePairData s = case lines s of
-	[] -> Nothing
-	(ha:l) -> do
-		addrs <- mapM unformatP2PAddress l
-		return (PairData (HalfAuthToken (T.pack ha)) addrs)
+deserializePairData :: [String] -> Maybe PairData
+deserializePairData [] = Nothing
+deserializePairData (ha:l) = do
+	addrs <- mapM unformatP2PAddress l
+	return (PairData (HalfAuthToken (T.pack ha)) addrs)
 
 data PairingResult
 	= PairSuccess
@@ -220,12 +219,12 @@ wormholePairing remotename ouraddrs ui = do
 	-- files. Permissions of received files may allow others
 	-- to read them. So, set up a temp directory that only
 	-- we can read.
-	withTmpDir "pair" $ \tmp -> do
-		liftIO $ void $ tryIO $ modifyFileMode (toRawFilePath tmp) $ 
+	withTmpDir (literalOsPath "pair") $ \tmp -> do
+		liftIO $ void $ tryIO $ modifyFileMode tmp $ 
 			removeModes otherGroupModes
-		let sendf = tmp </> "send"
-		let recvf = tmp </> "recv"
-		liftIO $ writeFileProtected (toRawFilePath sendf) $
+		let sendf = tmp </> literalOsPath "send"
+		let recvf = tmp </> literalOsPath "recv"
+		liftIO $ writeFileProtected sendf $
 			serializePairData ourpairdata
 
 		observer <- liftIO Wormhole.mkCodeObserver
@@ -235,23 +234,24 @@ wormholePairing remotename ouraddrs ui = do
 		-- the same channels that other wormhole users use.
 		let appid = Wormhole.appId "git-annex.branchable.com/p2p-setup"
 		(sendres, recvres) <- liftIO $
-			Wormhole.sendFile sendf observer appid
+			Wormhole.sendFile (fromOsPath sendf) observer appid
 				`concurrently`
-			Wormhole.receiveFile recvf producer appid
-		liftIO $ removeWhenExistsWith R.removeLink (toRawFilePath sendf)
+			Wormhole.receiveFile (fromOsPath recvf) producer appid
+		liftIO $ removeWhenExistsWith removeFile sendf
 		if sendres /= True
 			then return SendFailed
 			else if recvres /= True
 				then return ReceiveFailed
 				else do
 					r <- liftIO $ tryIO $
-						readFileStrict recvf
+						map decodeBS . fileLines'
+							<$> F.readFile' recvf
 					case r of
 						Left _e -> return ReceiveFailed
-						Right s -> maybe 
+						Right ls -> maybe 
 							(return ReceiveFailed)
 							(finishPairing 100 remotename ourhalf)
-							(deserializePairData s)
+							(deserializePairData ls)
 
 -- | Allow the peer we're pairing with to authenticate to us,
 -- using an authtoken constructed from the two HalfAuthTokens.
