@@ -6,6 +6,7 @@
  -}
 
 {-# LANGUAGE CPP #-}
+{-# LANGUAGE OverloadedStrings #-}
 
 module Git.Hook where
 
@@ -14,15 +15,14 @@ import Git
 import Utility.Tmp
 import Utility.Shell
 import Utility.FileMode
+import qualified Utility.FileIO as F
 #ifndef mingw32_HOST_OS
 import qualified Utility.RawFilePath as R
 import System.PosixCompat.Files (fileMode)
 #endif
 
-import qualified Data.ByteString as B
-
 data Hook = Hook
-	{ hookName :: FilePath
+	{ hookName :: OsPath
 	, hookScript :: String
 	, hookOldScripts :: [String]
 	}
@@ -31,8 +31,8 @@ data Hook = Hook
 instance Eq Hook where
 	a == b = hookName a == hookName b
 
-hookFile :: Hook -> Repo -> FilePath
-hookFile h r = fromRawFilePath (localGitDir r) </> "hooks" </> hookName h
+hookFile :: Hook -> Repo -> OsPath
+hookFile h r = localGitDir r </> literalOsPath "hooks" </> hookName h
 
 {- Writes a hook. Returns False if the hook already exists with a different
  - content. Upgrades old scripts.
@@ -58,15 +58,13 @@ hookWrite h r = ifM (doesFileExist f)
   where
 	f = hookFile h r
 	go = do
-		-- On Windows, using B.writeFile here avoids
-		-- the newline translation done by writeFile.
+		-- On Windows, using a ByteString as the file content
+		-- avoids the newline translation done by writeFile.
 		-- Hook scripts on Windows could use CRLF endings, but
 		-- they typically use unix newlines, which does work there
 		-- and makes the repository more portable.
-		viaTmp B.writeFile f (encodeBS (hookScript h))
-		void $ tryIO $ modifyFileMode
-			(toRawFilePath f)
-			(addModes executeModes)
+		viaTmp F.writeFile' f (encodeBS (hookScript h))
+		void $ tryIO $ modifyFileMode f (addModes executeModes)
 		return True
 
 {- Removes a hook. Returns False if the hook contained something else, and
@@ -91,7 +89,7 @@ expectedContent h r = do
 	-- and so a hook file that has CRLF will be treated the same as one
 	-- that has LF. That is intentional, since users may have a reason
 	-- to prefer one or the other.
-	content <- readFile $ hookFile h r
+	content <- readFile $ fromOsPath $ hookFile h r
 	return $ if content == hookScript h
 		then ExpectedContent
 		else if any (content ==) (hookOldScripts h)
@@ -103,13 +101,13 @@ hookExists h r = do
 	let f = hookFile h r
 	catchBoolIO $
 #ifndef mingw32_HOST_OS
-		isExecutable . fileMode <$> R.getFileStatus (toRawFilePath f)
+		isExecutable . fileMode <$> R.getFileStatus (fromOsPath f)
 #else
 		doesFileExist f
 #endif
 
-runHook :: Hook -> Repo -> IO Bool
-runHook h r = do
+runHook :: (FilePath -> [CommandParam] -> IO a) -> Hook -> [CommandParam] -> Repo -> IO a
+runHook runner h ps r = do
 	let f = hookFile h r
-	(c, ps) <- findShellCommand f
-	boolSystem c ps
+	(c, cps) <- findShellCommand f
+	runner c (cps ++ ps)
