@@ -19,6 +19,7 @@ import Annex.Common
 import Utility.TimeStamp
 import Logs.File
 import Types.RepoVersion
+import qualified Utility.FileIO as F
 
 import Data.Time.Clock.POSIX
 
@@ -31,14 +32,14 @@ writeUpgradeLog v t = do
 
 readUpgradeLog :: Annex [(RepoVersion, POSIXTime)]
 readUpgradeLog = do
-	logfile <- fromRawFilePath <$> fromRepo gitAnnexUpgradeLog
+	logfile <- fromRepo gitAnnexUpgradeLog
 	ifM (liftIO $ doesFileExist logfile)
-		( mapMaybe parse . lines
-			<$> liftIO (readFileStrict logfile)
+		( mapMaybe (parse . decodeBS) . fileLines'
+			<$> liftIO (F.readFile' logfile)
 		, return []
 		)
   where
-	parse line = case (readish sint, parsePOSIXTime ts) of
+	parse line = case (readish sint, parsePOSIXTime (encodeBS ts)) of
 		(Just v, Just t) -> Just (RepoVersion v, t)
 		_ -> Nothing
 	  where
