@@ -21,6 +21,7 @@ import Messages.Internal
 import Messages.Progress
 import qualified Messages.JSON as JSON
 import Utility.Metered (BytesProcessed, setMeterTotalSize)
+import Git.Quote
 
 import Control.Monad.IO.Class (MonadIO)
 
@@ -51,10 +52,11 @@ relaySerializedOutput getso sendsor meterreport runannex = go Nothing
 			runannex $ outputMessage'
 				(\_ _ -> return False)
 				id
-				msg
+				id
+				(UnquotedByteString msg)
 			loop st
 		Left (OutputError msg) -> do
-			runannex $ outputError msg
+			runannex $ outputError id $ UnquotedString msg
 			loop st		
 		Left (JSONObject b) -> do
 			runannex $ withMessageState $ \s -> case outputType s of
