@@ -49,6 +49,7 @@ import Logs.Cluster.Basic
 import Utility.Metered
 import Utility.Env
 import Utility.Batch
+import qualified Utility.FileIO as F
 import Remote.Helper.Git
 import Remote.Helper.Messages
 import Remote.Helper.ExportImport
@@ -98,8 +99,9 @@ locationField = Accepted "location"
 
 list :: Bool -> Annex [Git.Repo]
 list autoinit = do
-	c <- fromRepo Git.config
-	rs <- mapM (tweakurl c) =<< Annex.getGitRemotes
+	cfg <- fromRepo Git.config
+	fullcfg <- fromRepo Git.fullconfig
+	rs <- mapM (tweakurl cfg fullcfg) =<< Annex.getGitRemotes
 	rs' <- mapM (configRead autoinit) (filter (not . isGitRemoteAnnex) rs)
 	proxies <- doQuietAction getProxies
 	if proxies == mempty
@@ -108,17 +110,20 @@ list autoinit = do
 			proxied <- listProxied proxies rs'
 			return (proxied++rs')
   where
-	tweakurl c r = do
+	tweakurl cfg fullcfg r = do
 		let n = fromJust $ Git.remoteName r
-		case getAnnexUrl r c of
-			Just url | not (isP2PHttpProtocolUrl url) -> 
+		case getAnnexUrl r cfg fullcfg of
+			Just url | not (isP2PHttpProtocolUrl url) ->
 				inRepo $ \g -> Git.Construct.remoteNamed n $
 					Git.Construct.fromRemoteLocation url
 						False g
 			_ -> return r
 
-getAnnexUrl :: Git.Repo -> M.Map Git.ConfigKey Git.ConfigValue -> Maybe String
-getAnnexUrl r c = Git.fromConfigValue <$> M.lookup (annexUrlConfigKey r) c
+getAnnexUrl :: Git.Repo -> Git.RepoConfig -> Git.RepoFullConfig -> Maybe String
+getAnnexUrl r cfg fullcfg = 
+	(Git.fromConfigValue <$> M.lookup (annexUrlConfigKey r) cfg)
+		<|>
+	annexInsteadOfUrl fullcfg (Git.repoLocation r)
 
 annexUrlConfigKey :: Git.Repo -> Git.ConfigKey
 annexUrlConfigKey r = remoteConfig r "annexurl"
@@ -137,11 +142,11 @@ isGitRemoteAnnex r = "annex::" `isPrefixOf` Git.repoLocation r
  - etc.
  -}
 gitSetup :: SetupStage -> Maybe UUID -> Maybe CredPair -> RemoteConfig -> RemoteGitConfig -> Annex (RemoteConfig, UUID)
-gitSetup Init mu _ c _ = do
+gitSetup Init mu _ c gc = do
 	let location = maybe (giveup "Specify location=url") fromProposedAccepted $
 		M.lookup locationField c
 	r <- inRepo $ Git.Construct.fromRemoteLocation location False
-	r' <- tryGitConfigRead False r False
+	r' <- tryGitConfigRead gc False r False
 	let u = getUncachedUUID r'
 	if u == NoUUID
 		then giveup "git repository does not have an annex uuid"
@@ -182,10 +187,10 @@ configRead autoinit r = do
 	case (repoCheap r, annexignore, hasuuid) of
 		(_, True, _) -> return r
 		(True, _, _)
-			| remoteAnnexCheckUUID gc -> tryGitConfigRead autoinit r hasuuid
+			| remoteAnnexCheckUUID gc -> tryGitConfigRead gc autoinit r hasuuid
 			| otherwise -> return r
 		(False, _, False) -> configSpecialGitRemotes r >>= \case
-			Nothing -> tryGitConfigRead autoinit r False
+			Nothing -> tryGitConfigRead gc autoinit r False
 			Just r' -> return r'
 		_ -> return r
 
@@ -268,20 +273,31 @@ unavailable r u c gc = gen r' u c gc'
 
 {- Tries to read the config for a specified remote, updates state, and
  - returns the updated repo. -}
-tryGitConfigRead :: Bool -> Git.Repo -> Bool -> Annex Git.Repo
-tryGitConfigRead autoinit r hasuuid
+tryGitConfigRead :: RemoteGitConfig -> Bool -> Git.Repo -> Bool -> Annex Git.Repo
+tryGitConfigRead gc autoinit r hasuuid
 	| haveconfig r = return r -- already read
 	| Git.repoIsSsh r = storeUpdatedRemote $ do
 		v <- Ssh.onRemote NoConsumeStdin r
 			( pipedconfig Git.Config.ConfigList autoinit (Git.repoDescribe r)
-			, return (Left "configlist failed")
+			, error "internal"
 			)
 			"configlist" [] configlistfields
 		case v of
 			Right r'
 				| haveconfig r' -> return r'
-				| otherwise -> configlist_failed
-			Left _ -> configlist_failed
+				| otherwise -> do
+					configlist_failed
+					return r
+			Left exitcode -> do
+				-- ssh exits 255 when there was an error
+				-- connecting to the remote server.
+				if exitcode /= ExitFailure 255 
+					then do
+						configlist_failed
+						return r
+					else do
+						warning $ UnquotedString $ "Unable to connect to repository " ++ Git.repoDescribe r ++ " to get its annex.uuid configuration."
+						return r
 	| Git.repoIsHttp r = storeUpdatedRemote geturlconfig
 	| Git.GCrypt.isEncrypted r = handlegcrypt =<< getConfigMaybe (remoteAnnexConfig r "uuid")
 	| Git.repoIsUrl r = do
@@ -294,31 +310,35 @@ tryGitConfigRead autoinit r hasuuid
 	haveconfig = not . M.null . Git.config
 
 	pipedconfig st mustincludeuuuid configloc cmd params = do
-		v <- liftIO $ Git.Config.fromPipe r cmd params st
-		case v of
-			Right (r', val, _err) -> do
+		(r', val, exitcode, _err) <- liftIO $ 
+			Git.Config.fromPipe r cmd params st
+		if exitcode == ExitSuccess
+			then do
 				unless (isUUIDConfigured r' || val == mempty || not mustincludeuuuid) $ do
 					warning $ UnquotedString $ "Failed to get annex.uuid configuration of repository " ++ Git.repoDescribe r
 					warning $ UnquotedString $ "Instead, got: " ++ show val
 					warning "This is unexpected; please check the network transport!"
 				return $ Right r'
-			Left l -> do
+			else do
 				warning $ UnquotedString $ "Unable to parse git config from " ++ configloc
-				return $ Left (show l)
+				return $ Left exitcode
 
-	geturlconfig = Url.withUrlOptionsPromptingCreds $ \uo -> do
+	geturlconfig = Url.withUrlOptionsPromptingCreds (Just gc) $ \uo -> do
 		let url = Git.repoLocation r ++ "/config"
-		v <- withTmpFile "git-annex.tmp" $ \tmpfile h -> do
+		v <- withTmpFile (literalOsPath "git-annex.tmp") $ \tmpfile h -> do
 			liftIO $ hClose h
 			Url.download' nullMeterUpdate Nothing url tmpfile uo >>= \case
-				Right () -> pipedconfig Git.Config.ConfigNullList
-					False url "git"
-					[ Param "config"
-					, Param "--null"
-					, Param "--list"
-					, Param "--file"
-					, File tmpfile
-					]
+				Right () ->
+					pipedconfig Git.Config.ConfigNullList
+						False url "git"
+						[ Param "config"
+						, Param "--null"
+						, Param "--list"
+						, Param "--file"
+						, File (fromOsPath tmpfile)
+						] >>= return . \case
+							Right r' -> Right r'
+							Left exitcode -> Left $ "git config exited " ++ show exitcode
 				Left err -> return (Left err)
 		case v of
 			Right r' -> do
@@ -338,15 +358,7 @@ tryGitConfigRead autoinit r hasuuid
 				warning $ UnquotedString $ url ++ " " ++ err
 				return r
 
-	{- Is this remote just not available, or does
-	 - it not have git-annex-shell?
-	 - Find out by trying to fetch from the remote. -}
-	configlist_failed = case Git.remoteName r of
-		Nothing -> return r
-		Just n -> do
-			whenM (inRepo $ Git.Command.runBool [Param "fetch", Param "--quiet", Param n]) $ do
-				set_ignore "does not have git-annex installed" True
-			return r
+	configlist_failed = set_ignore "does not have git-annex installed" True
 	
 	set_ignore msg longmessage = do
 		case Git.remoteName r of
@@ -437,7 +449,7 @@ inAnnex' repo rmt st@(State connpool duc _ _ _) key
 	checkp2phttp = p2pHttpClient rmt giveup (clientCheckPresent key)
 	checkhttp = do
 		gc <- Annex.getGitConfig
-		Url.withUrlOptionsPromptingCreds $ \uo -> 
+		Url.withUrlOptionsPromptingCreds (Just (gitconfig rmt)) $ \uo -> 
 			anyM (\u -> Url.checkBoth u (fromKey keySize key) uo)
 				(keyUrls gc repo rmt key)
 	checkremote = P2PHelper.checkpresent (Ssh.runProto rmt connpool (cantCheck rmt)) key
@@ -458,9 +470,9 @@ keyUrls gc repo r key = map tourl locs'
 		| remoteAnnexBare remoteconfig == Just False = annexLocationsNonBare gc key
 		| otherwise = annexLocationsBare gc key
 #ifndef mingw32_HOST_OS
-	locs' = map fromRawFilePath locs
+	locs' = map fromOsPath locs
 #else
-	locs' = map (replace "\\" "/" . fromRawFilePath) locs
+	locs' = map (replace "\\" "/" . fromOsPath) locs
 #endif
 	remoteconfig = gitconfig r
 
@@ -548,17 +560,17 @@ lockKey' repo r st@(State connpool duc _ _ _) key callback
 	failedlock = giveup "can't lock content"
 
 {- Tries to copy a key's content from a remote's annex to a file. -}
-copyFromRemote :: Remote -> State -> Key -> AssociatedFile -> FilePath -> MeterUpdate -> VerifyConfig -> Annex Verification
+copyFromRemote :: Remote -> State -> Key -> AssociatedFile -> OsPath -> MeterUpdate -> VerifyConfig -> Annex Verification
 copyFromRemote r st key file dest meterupdate vc = do
 	repo <- getRepo r
 	copyFromRemote'' repo r st key file dest meterupdate vc
 
-copyFromRemote'' :: Git.Repo -> Remote -> State -> Key -> AssociatedFile -> FilePath -> MeterUpdate -> VerifyConfig -> Annex Verification
+copyFromRemote'' :: Git.Repo -> Remote -> State -> Key -> AssociatedFile -> OsPath -> MeterUpdate -> VerifyConfig -> Annex Verification
 copyFromRemote'' repo r st@(State connpool _ _ _ _) key af dest meterupdate vc
 	| isP2PHttp r = copyp2phttp
 	| Git.repoIsHttp repo = verifyKeyContentIncrementally vc key $ \iv -> do
 		gc <- Annex.getGitConfig
-		ok <- Url.withUrlOptionsPromptingCreds $
+		ok <- Url.withUrlOptionsPromptingCreds (Just (gitconfig r)) $
 			Annex.Content.downloadUrl False key meterupdate iv (keyUrls gc repo r key) dest
 		unless ok $
 			giveup "failed to download content"
@@ -591,9 +603,8 @@ copyFromRemote'' repo r st@(State connpool _ _ _ _) key af dest meterupdate vc
 		<|> remoteAnnexBwLimit (gitconfig r)
 		
 	copyp2phttp = verifyKeyContentIncrementally vc key $ \iv -> do
-		startsz <- liftIO $ tryWhenExists $
-			getFileSize (toRawFilePath dest)
-		bracketIO (openBinaryFile dest ReadWriteMode) (hClose) $ \h -> do
+		startsz <- liftIO $ tryWhenExists $ getFileSize dest
+		bracketIO (F.openBinaryFile dest ReadWriteMode) (hClose) $ \h -> do
 			metered (Just meterupdate) key bwlimit $ \_ p -> do
 				p' <- case startsz of
 					Just startsz' -> liftIO $ do
@@ -605,16 +616,18 @@ copyFromRemote'' repo r st@(State connpool _ _ _ _) key af dest meterupdate vc
 					Valid -> return ()
 					Invalid -> giveup "Transfer failed"
 
-copyFromRemoteCheap :: State -> Git.Repo -> Maybe (Key -> AssociatedFile -> FilePath -> Annex ())
+copyFromRemoteCheap :: State -> Git.Repo -> Maybe (Key -> AssociatedFile -> OsPath -> Annex ())
 #ifndef mingw32_HOST_OS
 copyFromRemoteCheap st repo
 	| not $ Git.repoIsUrl repo = Just $ \key _af file -> guardUsable repo (giveup "cannot access remote") $ do
 		gc <- getGitConfigFromState st
 		loc <- liftIO $ gitAnnexLocation key repo gc
-		liftIO $ ifM (R.doesPathExist loc)
+		liftIO $ ifM (doesFileExist loc)
 			( do
 				absloc <- absPath loc
-				R.createSymbolicLink absloc (toRawFilePath file)
+				R.createSymbolicLink
+					(fromOsPath absloc)
+					(fromOsPath file)
 			, giveup "remote does not contain key"
 			)
 	| otherwise = Nothing
@@ -623,12 +636,12 @@ copyFromRemoteCheap _ _ = Nothing
 #endif
 
 {- Tries to copy a key's content to a remote's annex. -}
-copyToRemote :: Remote -> State -> Key -> AssociatedFile -> Maybe FilePath -> MeterUpdate -> Annex ()
+copyToRemote :: Remote -> State -> Key -> AssociatedFile -> Maybe OsPath -> MeterUpdate -> Annex ()
 copyToRemote r st key af o meterupdate = do
 	repo <- getRepo r
 	copyToRemote' repo r st key af o meterupdate
 
-copyToRemote' :: Git.Repo -> Remote -> State -> Key -> AssociatedFile -> Maybe FilePath -> MeterUpdate -> Annex ()
+copyToRemote' :: Git.Repo -> Remote -> State -> Key -> AssociatedFile -> Maybe OsPath -> MeterUpdate -> Annex ()
 copyToRemote' repo r st@(State connpool duc _ _ _) key af o meterupdate
 	| isP2PHttp r = prepsendwith copyp2phttp
 	| not $ Git.repoIsUrl repo = ifM duc
@@ -669,9 +682,9 @@ copyToRemote' repo r st@(State connpool duc _ _ _) key af o meterupdate
 				let checksuccess = liftIO checkio >>= \case
 					Just err -> giveup err
 					Nothing -> return True
-				logStatusAfter NoLiveUpdate key $ Annex.Content.getViaTmp rsp verify key af (Just sz) $ \dest ->
+				logStatusAfter NoLiveUpdate key $ Annex.Content.getViaTmp rsp verify key (Just sz) $ \dest ->
 					metered (Just (combineMeterUpdate meterupdate p)) key bwlimit $ \_ p' -> 
-						copier object (fromRawFilePath dest) key p' checksuccess verify
+						copier object dest key p' checksuccess verify
 			)
 		unless res $
 			failedsend
@@ -707,10 +720,12 @@ fsckOnRemote r params
 		r' <- Git.Config.read r
 		environ <- getEnvironment
 		let environ' = addEntries 
-			[ ("GIT_WORK_TREE", fromRawFilePath $ Git.repoPath r')
-			, ("GIT_DIR", fromRawFilePath $ Git.localGitDir r')
+			[ ("GIT_WORK_TREE", fromOsPath $ Git.repoPath r')
+			, ("GIT_DIR", fromOsPath $ Git.localGitDir r')
 			] environ
-		batchCommandEnv program (Param "fsck" : params) (Just environ')
+		batchCommandEnv (fromOsPath program)
+			(Param "fsck" : params)
+			(Just environ')
 
 {- The passed repair action is run in the Annex monad of the remote. -}
 repairRemote :: Git.Repo -> Annex Bool -> Annex (IO Bool)
@@ -804,7 +819,7 @@ wantHardLink = (annexHardLink <$> Annex.getGitConfig)
 	-- because they can be modified at any time.
 	<&&> (not <$> annexThin <$> Annex.getGitConfig)
 
-type FileCopier = FilePath -> FilePath -> Key -> MeterUpdate -> Annex Bool -> VerifyConfig -> Annex (Bool, Verification)
+type FileCopier = OsPath -> OsPath -> Key -> MeterUpdate -> Annex Bool -> VerifyConfig -> Annex (Bool, Verification)
 
 -- If either the remote or local repository wants to use hard links,
 -- the copier will do so (falling back to copying if a hard link cannot be
@@ -817,14 +832,14 @@ type FileCopier = FilePath -> FilePath -> Key -> MeterUpdate -> Annex Bool -> Ve
 mkFileCopier :: Bool -> State -> Annex FileCopier
 mkFileCopier remotewanthardlink (State _ _ copycowtried _ _) = do
 	localwanthardlink <- wantHardLink
-	let linker = \src dest -> R.createLink (toRawFilePath src) (toRawFilePath dest) >> return True
+	let linker = \src dest -> R.createLink (fromOsPath src) (fromOsPath dest) >> return True
 	if remotewanthardlink || localwanthardlink
 		then return $ \src dest k p check verifyconfig ->
 			ifM (liftIO (catchBoolIO (linker src dest)))
 				( ifM check
 					( return (True, Verified)
 					, do
-						verificationOfContentFailed (toRawFilePath dest)
+						verificationOfContentFailed dest
 						return (False, UnVerified)
 					)
 				, copier src dest k p check verifyconfig
@@ -837,7 +852,7 @@ mkFileCopier remotewanthardlink (State _ _ copycowtried _ _) = do
 			Copied -> ifM check
 				( finishVerifyKeyContentIncrementally iv
 				, do
-					verificationOfContentFailed (toRawFilePath dest)
+					verificationOfContentFailed dest
 					return (False, UnVerified)
 				)
 			CopiedCoW -> unVerified check
@@ -875,7 +890,7 @@ mkState r u gc = do
 			rv <- liftIO newEmptyMVar
 			let getrepo = ifM (liftIO $ isEmptyMVar rv)
 				( do
-					r' <- tryGitConfigRead False r True
+					r' <- tryGitConfigRead gc False r True
 					let t = (r', extractGitConfig FromGitConfig r')
 					void $ liftIO $ tryPutMVar rv t
 					return t
