@@ -29,10 +29,10 @@ transport rr@(RemoteRepo r _) url h ichan ochan = do
 		Just (cmd, params) -> transportUsingCmd cmd params rr url h ichan ochan
 
 transportUsingCmd :: FilePath -> [CommandParam] -> Transport
-transportUsingCmd cmd params rr@(RemoteRepo r gc) url h@(TransportHandle (LocalRepo g) s) ichan ochan = do
+transportUsingCmd cmd params rr@(RemoteRepo r gc) url h@(TransportHandle (LocalRepo g) st rd) ichan ochan = do
 	-- enable ssh connection caching wherever inLocalRepo is called
 	g' <- liftAnnex h $ sshOptionsTo r gc g
-	let transporthandle = TransportHandle (LocalRepo g') s
+	let transporthandle = TransportHandle (LocalRepo g') st rd
 	transportUsingCmd' cmd params rr url transporthandle ichan ochan
 
 transportUsingCmd' :: FilePath -> [CommandParam] -> Transport
@@ -42,15 +42,13 @@ transportUsingCmd' cmd params (RemoteRepo r gc) url transporthandle ichan ochan
 	p = (proc cmd (toCommand params))
 		{ std_in = CreatePipe
 		, std_out = CreatePipe
-		, std_err = CreatePipe
 		}
 
-	go (Just toh) (Just fromh) (Just errh) pid = do
+	go (Just toh) (Just fromh) Nothing pid = do
 		-- Run all threads until one finishes and get the status
 		-- of the first to finish. Cancel the rest.
 		status <- catchDefaultIO (Right ConnectionClosed) $
-			handlestderr errh
-				`race` handlestdout fromh
+				handlestdout fromh
 					`race` handlecontrol
 
 		send (DISCONNECTED url)
@@ -58,7 +56,7 @@ transportUsingCmd' cmd params (RemoteRepo r gc) url transporthandle ichan ochan
 		hClose fromh
 		void $ waitForProcess pid
 
-		return $ either (either id id) id status
+		return $ either id id status
 	go _ _ _ _ = error "internal"
 
 	send msg = atomically $ writeTChan ochan msg
@@ -88,21 +86,3 @@ transportUsingCmd' cmd params (RemoteRepo r gc) url transporthandle ichan ochan
 			STOP -> return ConnectionStopping
 			LOSTNET -> return ConnectionStopping
 			_ -> handlecontrol
-
-	-- Old versions of git-annex-shell that do not support
-	-- the notifychanges command will exit with a not very useful
-	-- error message. Detect that error, and avoid reconnecting.
-	-- Propigate all stderr.
-	handlestderr errh = do
-		s <- hGetSomeString errh 1024
-		hPutStr stderr s
-		hFlush stderr
-		if "git-annex-shell: git-shell failed" `isInfixOf` s
-			then do
-				send $ WARNING url $ unwords
-					[ "Remote", Git.repoDescribe r
-					, "needs its git-annex upgraded"
-					, "to 5.20140405 or newer"
-					]
-				return ConnectionStopping
-			else handlestderr errh
