@@ -1,6 +1,6 @@
 {- Enumeration of cleanup actions
  -
- - Copyright 2014 Joey Hess <id@joeyh.name>
+ - Copyright 2014-2020 Joey Hess <id@joeyh.name>
  -
  - Licensed under the GNU AGPL version 3 or higher.
  -}
@@ -8,9 +8,10 @@
 module Types.CleanupActions where
 
 import Types.UUID
-
 import Utility.Url
 
+import System.Process (Pid)
+
 data CleanupAction
 	= RemoteCleanup UUID
 	| StopHook UUID
@@ -20,3 +21,7 @@ data CleanupAction
 	| TorrentCleanup URLString
 	| OtherTmpCleanup
 	deriving (Eq, Ord)
+
+data SignalAction
+	= PropagateSignalProcessGroup Pid
+	deriving (Eq, Ord)
